var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
import { html, nothing } from "lit";
import { property, query, queryAll, queryAssignedElements, state } from "lit/decorators.js";
import { repeat } from "lit/directives/repeat.js";
import { classMap } from "lit/directives/class-map.js";
import { ifDefined } from "lit/directives/if-defined.js";
import { computeFlipOffsetPosition, FtLitElement, noTextInputDefaultClearButton, toFtFormComponent } from "@fluid-topics/ft-wc-utils";
import { FtTypography, FtTypographyBody1 } from "@fluid-topics/ft-typography";
import { FtInputLabel } from "@fluid-topics/ft-input-label";
import { FtRipple } from "@fluid-topics/ft-ripple";
import { FtIcon, FtIcons } from "@fluid-topics/ft-icon";
import { FtTextFieldCssVariables, styles } from "./ft-text-field.styles";
import { FtTextFieldSuggestion } from "./ft-text-field-suggestion";
class FtTextField extends toFtFormComponent(FtLitElement, "textbox") {
    constructor() {
        super(...arguments);
        this._value = "";
        this.dispatchedValue = "";
        this.outlined = false;
        this.disabled = false;
        this.error = false;
        this.fixedMenuPosition = false;
        this.prefix = null;
        this.passwordHiddenIcon = FtIcons.EYE_SLASH;
        this.passwordRevealedIcon = FtIcons.EYE;
        this.filterSuggestions = false;
        this.password = false;
        this.focused = false;
        this.hidePassword = true;
        this.hideSuggestions = false;
        this.visibleSuggestions = [];
        this.providedSuggestions = [];
    }
    /*
     * We prevent lit from creating setter and getter to differentiate
     * when the value change comes from inside or outside the component
     * so that we can update the reference to the last dispatched value in consequence.
     * Lit will use the setter that updates the reference
     * and the component should use `setInternalValue` that does not.
     * */
    get value() {
        return this._value || "";
    }
    set value(value) {
        this.setInternalValue(value || "");
        this.dispatchedValue = value || "";
    }
    setInternalValue(value) {
        const oldValue = this._value;
        this._value = value || "";
        this.setFormValue(this._value);
        this.requestUpdate("value", oldValue);
    }
    focus() {
        var _a;
        (_a = this.input) === null || _a === void 0 ? void 0 : _a.focus();
    }
    render() {
        const classes = {
            "ft-text-field": true,
            "ft-text-field--filled": !this.outlined,
            "ft-text-field--outlined": this.outlined,
            "ft-text-field--disabled": this.disabled,
            "ft-text-field--has-value": !!this.value,
            "ft-text-field--with-label": !!this.label,
            "ft-text-field--in-error": this.error,
            "ft-text-field--fixed": this.fixedMenuPosition,
            "ft-text-field--with-prefix": !!this.prefix,
            "ft-text-field--hide-suggestions": !this.suggestionsShouldBeDisplayed(),
            "ft-text-field--raised-label": this.focused || this.value != "",
            "ft-text-field--with-icon": !!this.icon,
            "ft-text-field--with-password": this.isPasswordField()
        };
        const suggestionsAreProvidedAndQueryNotInPossiblesValues = this.suggestionsProvider && this.value && this.value != "" && !this.providedSuggestions.map(p => p.label).includes(this.value);
        const suggestionsFromSlotAndQueryNotEmpty = !this.suggestionsProvider && this.filterSuggestions && this.value && this.value != "";
        const shouldDisplayQueryAsNewSuggestion = suggestionsFromSlotAndQueryNotEmpty || suggestionsAreProvidedAndQueryNotInPossiblesValues;
        return html `
            <div class="${classMap(classes)}">
                <div class="ft-text-field--main-panel"
                     @keydown=${this.handleKeyboardNavigation}
                     @focusout=${this.onMainPanelBlur}>
                    <ft-input-label text="${this.label}"
                                    ?disabled=${this.disabled}
                                    ?outlined=${this.outlined}
                                    ?raised=${this.focused || this.value != ""}
                                    ?error=${this.error}></ft-input-label>
                    <div class="ft-text-field--input-panel">
                        ${this.outlined ? nothing : html `
                            <ft-ripple ?disabled=${this.disabled} activated></ft-ripple>
                        `}
                        ${this.prefix ? html `
                                <ft-typography class="ft-text-field--prefix" variant="body1">
                                    ${this.prefix}
                                </ft-typography>
                            ` : nothing}
                        <input type=${this.resolveInputType()}
                               name=${ifDefined(this.name)}
                               maxlength=${ifDefined(this.maxLength || undefined)}
                               aria-label="${this.label}"
                               autocomplete=${ifDefined(this.autocomplete)}
                               class="ft-typography--body1 ft-text-field--input"
                               part="input"
                               ?disabled=${this.disabled}
                               .value=${this.value}
                               @click=${this.handleClick}
                               @keyup=${this.handleInput}
                               @focus=${this.onFocus}/>
                        ${(this.renderIcon())}
                    </div>
                    <div class="ft-text-field--suggestions"
                         @suggestion-selected=${this.onSuggestionSelected}>
                        ${this.renderSuggestions()}
                        ${shouldDisplayQueryAsNewSuggestion
            ? html `
                                <ft-text-field-suggestion class="ft-text-field-suggestion--new-value" helper="${this.suggestionsHelper}">
                                    ${this.value}
                                </ft-text-field-suggestion>
                            `
            : nothing}
                    </div>
                </div>
                ${this.helper ? html `
                    <ft-typography class="ft-text-field--helper-text" variant="caption">
                        ${this.helper}
                    </ft-typography>
                ` : nothing}
            </div>
        `;
    }
    resolveInputType() {
        var _a;
        if (this.isPasswordField()) {
            return this.hidePassword ? "password" : "text";
        }
        return (_a = this.type) !== null && _a !== void 0 ? _a : "text";
    }
    renderSuggestions() {
        if (this.suggestionsProvider) {
            return html `
                ${repeat(this.providedSuggestions, suggestion => html `
                        <ft-text-field-suggestion value=${suggestion.value} @click=${suggestion.clickHandler}>
                            ${suggestion.icon ? html `
                                                   <ft-icon .value=${suggestion.icon}></ft-icon>
                                               `
                : nothing}
                            ${suggestion.label}
                        </ft-text-field-suggestion>`)}
            `;
        }
        else {
            return html `
                <slot @slotchange=${() => this.filterSuggestionsIfNeeded()}></slot>
            `;
        }
    }
    renderPasswordIcon() {
        return html `
            <ft-icon class="ft-text-field--icon"
                     .variant=${this.iconVariant}
                     .value=${(this.hidePassword ? this.passwordHiddenIcon : this.passwordRevealedIcon)}
                     @click=${() => this.togglePasswordVisibility()}></ft-icon>
        `;
    }
    renderIcon() {
        if (this.isPasswordField()) {
            return this.renderPasswordIcon();
        }
        return this.icon ? html `
            <ft-icon class="ft-text-field--icon"
                     .variant=${this.iconVariant}
                     .value=${this.icon}
                     @click=${() => { var _a; return (_a = this.input) === null || _a === void 0 ? void 0 : _a.focus(); }}></ft-icon>
        ` : nothing;
    }
    update(props) {
        super.update(props);
        if (props.has("value")
            || props.has("filterSuggestions")
            || props.has("suggestionsProvider")) {
            this.updateSuggestions();
        }
        if (props.has("value") && props.get("value") != null) {
            this.hideSuggestions = false;
        }
        if (props.has("dispatchedValue") && props.get("dispatchedValue") != null) {
            this.hideSuggestions = true;
        }
    }
    async updateSuggestions() {
        if (this.suggestionsProvider) {
            this.providedSuggestions = await this.suggestionsProvider(this.value);
        }
        else {
            this.filterSuggestionsIfNeeded();
        }
    }
    filterSuggestionsIfNeeded() {
        if (this.filterSuggestions) {
            this.slottedSuggestions.forEach(s => s.hidden = !s.getValue().toLowerCase().includes(this.value.toLowerCase()));
            this.visibleSuggestions = this.slottedSuggestions.filter(s => !s.hidden);
        }
        else {
            this.visibleSuggestions = this.slottedSuggestions;
        }
        if (this.newValueSuggestion) {
            this.visibleSuggestions = [...this.visibleSuggestions, this.newValueSuggestion];
        }
        this.hideSuggestions = this.visibleSuggestions.length <= 0;
    }
    contentAvailableCallback(props) {
        super.contentAvailableCallback(props);
        if (props.has("focused") && !this.hideSuggestions && this.visibleSuggestions.length > 0) {
            this.setSuggestionPosition();
        }
        if (props.has("value") && this.suggestionsProvider) {
            this.visibleSuggestions = [...this.providedSuggestionsInDom];
        }
        if (props.has("autocomplete")) {
            this.setupWatchAutofill();
        }
        if (this.suggestionsShouldBeDisplayed()) {
            this.setSuggestionPosition();
        }
    }
    setupWatchAutofill() {
        var _a;
        this.clearWatchAutofill();
        if (((_a = this.autocomplete) !== null && _a !== void 0 ? _a : "off") !== "off") {
            this.watchAutofillInterval = setInterval(() => {
                var _a, _b, _c;
                if (((_a = this.input) === null || _a === void 0 ? void 0 : _a.matches(":autofill")) && this.dispatchedValue !== ((_b = this.input) === null || _b === void 0 ? void 0 : _b.value)) {
                    this.setValue((_c = this.input) === null || _c === void 0 ? void 0 : _c.value, true);
                }
            }, 200);
        }
    }
    clearWatchAutofill() {
        if (this.watchAutofillInterval) {
            clearInterval(this.watchAutofillInterval);
        }
    }
    connectedCallback() {
        super.connectedCallback();
        this.setupWatchAutofill();
    }
    disconnectedCallback() {
        super.disconnectedCallback();
        this.clearWatchAutofill();
    }
    setValue(newValue, fireChangeEvent = false) {
        if (this.value !== newValue) {
            this.setInternalValue(newValue);
            this.dispatchEvent(new CustomEvent("live-change", { detail: newValue }));
        }
        if (fireChangeEvent && this.dispatchedValue !== newValue) {
            this.dispatchedValue = newValue;
            this.dispatchEvent(new CustomEvent("change", { detail: newValue }));
        }
    }
    handleInput(event) {
        var _a;
        const newValue = ((_a = this.input) === null || _a === void 0 ? void 0 : _a.value) || "";
        this.setValue(newValue, event.key == "Escape" || event.key == "Enter");
    }
    handleClick() {
        this.hideSuggestions = false;
    }
    handleKeyboardNavigation(event) {
        var _a;
        if ((event.key === "ArrowDown" || event.key === "ArrowUp")) {
            event.preventDefault();
            event.stopPropagation();
            this.hideSuggestions = false;
            const currentSuggestion = this.visibleSuggestions.findIndex((element) => element.matches((":focus-within")));
            let target;
            if (event.key === "ArrowDown") {
                target = currentSuggestion < this.visibleSuggestions.length - 1 ? currentSuggestion + 1 : 0;
            }
            else {
                target = currentSuggestion > 0 ? currentSuggestion - 1 : this.visibleSuggestions.length - 1;
            }
            (_a = this.visibleSuggestions[target]) === null || _a === void 0 ? void 0 : _a.focus();
        }
        if (event.key == "Escape" || event.key == "Enter") {
            this.hideSuggestions = true;
        }
    }
    onSuggestionSelected(event) {
        var _a;
        this.setValue(event.detail, true);
        (_a = this.input) === null || _a === void 0 ? void 0 : _a.focus();
        setTimeout(() => this.hideSuggestions = true, 0);
    }
    onFocus() {
        this.focused = true;
        this.hideSuggestions = false;
    }
    onMainPanelBlur() {
        var _a, _b;
        if (!((_a = this.mainPanel) === null || _a === void 0 ? void 0 : _a.matches(":focus-within"))) {
            this.focused = false;
            this.setValue(((_b = this.input) === null || _b === void 0 ? void 0 : _b.value) || "", true);
        }
    }
    togglePasswordVisibility() {
        this.hidePassword = !this.hidePassword;
    }
    isPasswordField() {
        return this.password || this.type === "password";
    }
    setSuggestionPosition() {
        this.suggestionsContainer.style.width = this.mainPanel.getBoundingClientRect().width + "px";
        const fallbackPlacements = ["bottom", "top"];
        computeFlipOffsetPosition(this.mainPanel, this.suggestionsContainer, "bottom", fallbackPlacements, "fixed", FtTextFieldCssVariables.suggestSize.name, 0)
            .then(({ x, y }) => {
            this.suggestionsContainer.style.left = `${x}px`;
            this.suggestionsContainer.style.top = `${y}px`;
        });
    }
    suggestionsShouldBeDisplayed() {
        return !this.hideSuggestions && (this.visibleSuggestions.length || this.providedSuggestions.length);
    }
}
FtTextField.elementDefinitions = {
    "ft-input-label": FtInputLabel,
    "ft-ripple": FtRipple,
    "ft-typography": FtTypography,
    "ft-icon": FtIcon,
    "ft-text-field-suggestion": FtTextFieldSuggestion
};
// language=CSS
FtTextField.styles = [
    FtTypographyBody1,
    noTextInputDefaultClearButton,
    styles
];
__decorate([
    property()
], FtTextField.prototype, "label", void 0);
__decorate([
    property({ noAccessor: true })
], FtTextField.prototype, "value", null);
__decorate([
    state()
], FtTextField.prototype, "dispatchedValue", void 0);
__decorate([
    property()
], FtTextField.prototype, "helper", void 0);
__decorate([
    property()
], FtTextField.prototype, "suggestionsHelper", void 0);
__decorate([
    property({ type: Boolean })
], FtTextField.prototype, "outlined", void 0);
__decorate([
    property({ type: Boolean })
], FtTextField.prototype, "disabled", void 0);
__decorate([
    property({ type: Boolean })
], FtTextField.prototype, "error", void 0);
__decorate([
    property({ type: Boolean })
], FtTextField.prototype, "fixedMenuPosition", void 0);
__decorate([
    property()
], FtTextField.prototype, "prefix", void 0);
__decorate([
    property()
], FtTextField.prototype, "icon", void 0);
__decorate([
    property()
], FtTextField.prototype, "name", void 0);
__decorate([
    property()
], FtTextField.prototype, "type", void 0);
__decorate([
    property()
], FtTextField.prototype, "passwordHiddenIcon", void 0);
__decorate([
    property()
], FtTextField.prototype, "passwordRevealedIcon", void 0);
__decorate([
    property()
], FtTextField.prototype, "iconVariant", void 0);
__decorate([
    property({ type: Boolean })
], FtTextField.prototype, "filterSuggestions", void 0);
__decorate([
    property({ type: Number })
], FtTextField.prototype, "maxLength", void 0);
__decorate([
    property({ type: Boolean })
], FtTextField.prototype, "password", void 0);
__decorate([
    property({ type: String })
], FtTextField.prototype, "autocomplete", void 0);
__decorate([
    property({ attribute: false })
], FtTextField.prototype, "suggestionsProvider", void 0);
__decorate([
    state()
], FtTextField.prototype, "focused", void 0);
__decorate([
    state()
], FtTextField.prototype, "hidePassword", void 0);
__decorate([
    state()
], FtTextField.prototype, "hideSuggestions", void 0);
__decorate([
    state()
], FtTextField.prototype, "visibleSuggestions", void 0);
__decorate([
    state()
], FtTextField.prototype, "providedSuggestions", void 0);
__decorate([
    query(".ft-text-field--main-panel")
], FtTextField.prototype, "mainPanel", void 0);
__decorate([
    query(".ft-text-field--input")
], FtTextField.prototype, "input", void 0);
__decorate([
    query(".ft-text-field-suggestion--new-value")
], FtTextField.prototype, "newValueSuggestion", void 0);
__decorate([
    query(".ft-text-field--suggestions")
], FtTextField.prototype, "suggestionsContainer", void 0);
__decorate([
    queryAll("ft-text-field-suggestion")
], FtTextField.prototype, "providedSuggestionsInDom", void 0);
__decorate([
    queryAssignedElements({ selector: "ft-text-field-suggestion" })
], FtTextField.prototype, "slottedSuggestions", void 0);
export { FtTextField };
