import { css } from "lit";
import { designSystemVariables, FtCssVariableFactory, setVariable } from "@fluid-topics/ft-wc-utils";
import { FtTypographyBody1CssVariables } from "@fluid-topics/ft-typography/build/ft-typography.styles";
import { FtInputLabelCssVariables } from "@fluid-topics/ft-input-label/build/ft-input-label.styles";
import { FtRippleCssVariables } from "@fluid-topics/ft-ripple/build/ft-ripple.styles";
import { FtIconCssVariables } from "@fluid-topics/ft-icon/build/ft-icon.styles";
export const FtTextFieldCssVariables = {
    fontSize: FtCssVariableFactory.create("--ft-text-field-font-size", "", "SIZE", "14px"),
    labelSize: FtCssVariableFactory.create("--ft-text-field-label-size", "", "SIZE", "11px"),
    verticalSpacing: FtCssVariableFactory.create("--ft-text-field-vertical-spacing", "", "SIZE", "4px"),
    horizontalSpacing: FtCssVariableFactory.create("--ft-text-field-horizontal-spacing", "", "SIZE", "16px"),
    helperColor: FtCssVariableFactory.extend("--ft-text-field-helper-color", "", designSystemVariables.colorOnSurfaceMedium),
    colorPrimary: FtCssVariableFactory.external(designSystemVariables.colorPrimary, "Design system"),
    colorOnSurface: FtCssVariableFactory.external(designSystemVariables.colorOnSurface, "Design system"),
    colorOnSurfaceDisabled: FtCssVariableFactory.external(designSystemVariables.colorOnSurfaceDisabled, "Design system"),
    borderRadiusS: FtCssVariableFactory.external(designSystemVariables.borderRadiusS, "Design system"),
    colorError: FtCssVariableFactory.external(designSystemVariables.colorError, "Design system"),
    prefixColor: FtCssVariableFactory.extend("--ft-text-field-prefix-color", "", designSystemVariables.colorOnSurfaceMedium),
    iconColor: FtCssVariableFactory.extend("--ft-text-field-icon-color", "", designSystemVariables.colorOnSurfaceMedium),
    floatingZIndex: FtCssVariableFactory.create("--ft-text-field-floating-components-z-index", "", "NUMBER", "3"),
    backgroundColor: FtCssVariableFactory.extend("--ft-text-field-background-color", "", designSystemVariables.colorSurface),
    colorOutline: FtCssVariableFactory.external(designSystemVariables.colorOutline, "Design system"),
    elevation02: FtCssVariableFactory.external(designSystemVariables.elevation02, "Design system"),
    suggestSize: FtCssVariableFactory.create("--ft-text-field-suggest-size", "", "SIZE", "300px"),
};
// language=CSS
export const styles = css `
    *:focus {
        outline: none;
    }

    .ft-text-field {
        display: flex;
        flex-direction: column;
        align-items: stretch;
        position: relative;
    }

    ft-input-label {
        ${setVariable(FtInputLabelCssVariables.fontSize, FtTextFieldCssVariables.fontSize)};
        ${setVariable(FtInputLabelCssVariables.raisedFontSize, FtTextFieldCssVariables.labelSize)};
        ${setVariable(FtInputLabelCssVariables.verticalSpacing, FtTextFieldCssVariables.verticalSpacing)};
        ${setVariable(FtInputLabelCssVariables.horizontalSpacing, FtTextFieldCssVariables.horizontalSpacing)};
        ${setVariable(FtInputLabelCssVariables.backgroundColor, FtTextFieldCssVariables.backgroundColor)};
    }

    .ft-text-field--main-panel {
        position: relative;
        display: flex;
        height: calc(4 * ${FtTextFieldCssVariables.verticalSpacing} + ${FtTextFieldCssVariables.labelSize} + ${FtTextFieldCssVariables.fontSize});
    }

    .ft-text-field--input-panel {
        flex-grow: 1;
        position: relative;
        display: flex;
        align-items: center;
        overflow: hidden;
        padding: 0 ${FtTextFieldCssVariables.horizontalSpacing};

        ${setVariable(FtTypographyBody1CssVariables.fontSize, FtTextFieldCssVariables.fontSize)};
        ${setVariable(FtTypographyBody1CssVariables.lineHeight, FtTextFieldCssVariables.fontSize)};
    }

    .ft-text-field--input-panel ft-ripple {
        ${setVariable(FtRippleCssVariables.opacityContentOnSurfaceHover, "0.08")};
        ${setVariable(FtRippleCssVariables.opacityContentOnSurfacePressed, "0.04")};
    }

    .ft-text-field--filled.ft-text-field--with-label .ft-text-field--input-panel {
        align-items: flex-end;
        padding: 0 ${FtTextFieldCssVariables.horizontalSpacing} ${FtTextFieldCssVariables.verticalSpacing} ${FtTextFieldCssVariables.horizontalSpacing};
    }

    .ft-text-field--prefix {
        color: ${FtTextFieldCssVariables.prefixColor};
        margin-right: 2px;
        flex-shrink: 1;
        overflow: hidden;
        white-space: nowrap;
        text-overflow: ellipsis;
    }

    .ft-text-field--with-label:not(.ft-text-field--raised-label) .ft-text-field--prefix {
        display: none;
    }

    .ft-text-field--input {
        display: block;
        position: relative;
        flex-grow: 1;
        flex-shrink: 1;
        min-width: 0; /* flex sets this to auto an prevents input from shrinking properly */

        color: ${FtTextFieldCssVariables.colorOnSurface};
        padding: calc(2 * ${FtTextFieldCssVariables.verticalSpacing}) 0;
        border: none;
        background: none;

        &:-webkit-autofill,
        &:-webkit-autofill:hover,
        &:-webkit-autofill:focus {
            -webkit-text-fill-color: ${FtTextFieldCssVariables.colorOnSurface};
            transition: background-color 50000000s ease-in-out 100000s;
        }
    }

    .ft-text-field--filled.ft-text-field--with-label .ft-text-field--input {
        padding-bottom: 0;
        padding-top: calc(${FtTextFieldCssVariables.labelSize} + 2 * ${FtTextFieldCssVariables.verticalSpacing});
    }

    .ft-text-field--input::-webkit-calendar-picker-indicator,
    .ft-text-field--input::-webkit-list-button {
        display: none !important;
    }

    .ft-text-field--disabled .ft-text-field--input {
        color: ${FtTextFieldCssVariables.colorOnSurfaceDisabled};
    }

    .ft-text-field:not(.ft-text-field--disabled):focus-within ft-input-label {
        ${setVariable(FtInputLabelCssVariables.borderColor, FtTextFieldCssVariables.colorPrimary)};
        ${setVariable(FtInputLabelCssVariables.textColor, FtTextFieldCssVariables.colorPrimary)};
    }

    .ft-text-field--filled .ft-text-field--input-panel {
        border-radius: ${FtTextFieldCssVariables.borderRadiusS} ${FtTextFieldCssVariables.borderRadiusS} 0 0;
    }

    .ft-text-field--outlined .ft-text-field--input-panel {
        border-radius: ${FtTextFieldCssVariables.borderRadiusS};
    }

    .ft-text-field--helper-text {
        padding: 0 12px 0 ${FtTextFieldCssVariables.horizontalSpacing};
        color: ${FtTextFieldCssVariables.helperColor};
    }

    .ft-text-field--in-error .ft-text-field--input,
    .ft-text-field--in-error .ft-text-field--helper-text,
    .ft-text-field--in-error .ft-text-field--prefix {
        color: ${FtTextFieldCssVariables.colorError};
    }

    .ft-text-field--icon {
        align-self: center;
        margin-left: 8px;
        color: ${FtTextFieldCssVariables.iconColor};
    }

    .ft-text-field--container {
        position: relative;
    }

    .ft-text-field--suggestions {
        display: none;
        flex-direction: column;
        position: absolute;
        z-index: ${FtTextFieldCssVariables.floatingZIndex};
        background: ${FtTextFieldCssVariables.backgroundColor};
        border: 1px solid ${FtTextFieldCssVariables.colorOutline};
        border-radius: 0 0 ${FtTextFieldCssVariables.borderRadiusS} ${FtTextFieldCssVariables.borderRadiusS};
        box-shadow: ${FtTextFieldCssVariables.elevation02};
        outline: none;
        max-height: ${FtTextFieldCssVariables.suggestSize};
        overflow-y: auto;
    }

    .ft-text-field--fixed .ft-text-field--suggestions {
        position: fixed;
    }

    .ft-text-field:not(.ft-text-field--hide-suggestions):focus-within .ft-text-field--suggestions {
        display: flex;
    }

    .ft-text-field--with-icon {
        ${setVariable(FtInputLabelCssVariables.labelMaxWidth, `calc(100% - ${FtIconCssVariables.size} - ${FtInputLabelCssVariables.horizontalSpacing})`)};
    }

    .ft-text-field--with-password ft-icon:hover {
        cursor: pointer;
    }
`;
